import jax
import numpy as np

# todo: loop over all wavelengths.


def prep_polynomial(cross_section, wavelength_ind):
    pass


@jax.jit
def fit_polynomial(wavelength_ind, plot=False, species="H2", save=False):
    """
    todo: actually take in the Opac object.
    fits a polynomial to the opacity data.
    :param wavelength_ind:
    :param plot:
    :param species:
    :param save:
    :return:
    """
    xsecarr = species_dict[species]

    single_pres_single_temp = xsecarr[:, :, wavelength_ind].copy()
    X, Y = np.meshgrid(np.log10(T), np.log10(P), copy=False)
    #     X, Y = np.meshgrid(T, P, copy=False)
    Z = single_pres_single_temp

    X = X.flatten()
    Y = Y.flatten()

    # took out the y**2 term

    A = np.array(
        [
            X * 0 + 1,
            X,
            Y,
            X**2,
            X**2 * Y,
            X**2 * Y**2,
            X * Y**2,
            X * Y,
            X * Y**3,
            Y**4,
            Y**5,
            Y**6,
            Y**7,
            Y**12,
            X * Y**9,
            np.power(X, 1 / 4),
        ]
    ).T
    B = Z.flatten()

    coeff, r, rank, s = np.linalg.lstsq(A, B, rcond=-1)

    TT, PP = np.meshgrid(np.log10(T), np.log10(P), copy=False)

    z2 = (
        np.ones_like(TT),
        TT,
        PP,
        TT**2,
        TT**2 * PP,
        TT**2 * PP**2,
        TT * PP**2,
        TT * PP,
        TT * PP**3,
        PP**4,
        PP**5,
        PP**6,
        PP**7,
        PP**12,
        TT * PP**9,
        np.power(TT, 1 / 4),
    )

    im = np.tensordot(z2, coeff, axes=([0], [0]))

    return np.max(abs((im - Z) / Z)), np.mean(abs((im - Z) / Z)), coeff



"""
This file holds the classes for evaluating opacity data as trained by a neural network.

maybe call all of these eval_poly.py, eval_neural_net.py, etc.?

author: @arjunsavel
"""
import jax
import numpy as np
from jax import lax


@jax.jit
def feed_forward(x, n_layers, weights, biases):
    """
    feed forward neural network. this is a function that takes in the input, weights, and biases and returns the output.

    This can probably be optimized, to be honest.
    :param x:
    :param weights:
    :param biases:
    :return:
    """

    def inner_function(i, x):
        return jax.nn.sigmoid(x.dot(weights[i]) + biases[i])

    res = lax.fori_loop(0, n_layers, inner_function, x)

    return res


@jax.jit
def eval_neural_net(x, n_layers, weights, biases):
    res = feed_forward(x[0], n_layers, weights, biases)
    return res



import jax
import numpy as np

@jax.jit
def eval_polynomial(T, P, xsec, j):
    """
    evaluates the polynomial at a given temperature and pressure.
    :param T:
    :param P:
    :param xsec:
    :param j: the gas index to be used.
    :return:

    """
    TT = np.log10(T)
    PP = np.log10(P)

    z2 = (1.0, TT, PP, TT ** 2, TT ** 2 * PP, TT ** 2 * PP ** 2, TT * PP ** 2, TT * PP, TT * PP ** 3, PP ** 4, PP ** 5,
          PP ** 6, PP ** 7, PP ** 12, TT * PP ** 9, TT ** .25)
    Nterms = len(z2)

    xsec_int = 0.0
    for k in range(Nterms):
        xsec_int += z2[k] * xsec[k, j]
    xsec_int = 10 ** xsec_int  # we were keeping track of log opacity!

    return xsec_int